# -*- coding: utf-8 -*-
import pytz
import logging
import sqlite3

from sqlalchemy.ext.automap import automap_base
from sqlalchemy.orm import Session
from sqlalchemy import create_engine
from sqlalchemy.sql import text

from utils.config_loader import ConfigSectionMap
from utils.utils import print_info


class DbCredentials(object):

    def __init__(self):
        self.conn = sqlite3.connect('crendentials.db')
        self.cursor = self.conn.cursor()
        self.cursor.execute(
            '''CREATE TABLE IF NOT EXISTS credentials('''
            '''id INTEGER PRIMARY KEY AUTOINCREMENT, '''
            '''str_conexion text, server_receptor text, sql_lector text, '''
            '''lapse_seconds text, table_update text, id_name_update text, '''
            '''field_update text)''')

    def insert(self, **values):
        sql = (
            "insert or replace into credentials (str_conexion, "
            "server_receptor, sql_lector, lapse_seconds, table_update, "
            "id_name_update, field_update) values "
            "('%s', '%s', '%s', '%s', '%s', '%s', '%s')" % (
                values['str_conexion'],
                values['server_receptor'],
                values['sql_lector'],
                values['lapse_seconds'],
                values['table_update'],
                values['id_name_update'],
                values['field_update']
            )
        )
        self.cursor.execute("delete from credentials;")
        self.cursor.execute(sql)
        self.conn.commit()
        # self.conn.close()

    def dictfetchall(self, result):
        """
        Return all rows from a cursor as a dict"
        """

        columns = [
            'id',
            'str_conexion',
            'server_receptor',
            'sql_lector',
            'lapse_seconds',
            'table_update',
            'id_name_update',
            'field_update',
        ]

        return dict(zip(columns, result or ('', '', '', '', '', '', '', '')))

    def get(self):
        sql = "select * from credentials;"
        rows = self.cursor.execute(sql)
        rows = rows.fetchone()
        self.conn.commit()

        return self.dictfetchall(rows)

    def check_string_con(self, string):
        string = string.strip()
        try:
            first_split = string.rsplit('@', 1)
            first_part_first_split = first_split[0]
            if first_part_first_split == "":
                return False, None, None, None, None, None, None
            second_part_first_split = first_split[1]
            if second_part_first_split == "":
                return False, None, None, None, None, None, None
        except:
            return False, None, None, None, None, None, None

        try:
            second_split = first_part_first_split.split(':')
            engine = second_split[0]
            if engine == "":
                return False, None, None, None, None, None, None
            user = second_split[1]
            if user == "":
                return False, None, None, None, None, None, None
            if '//' not in user:
                return False, None, None, None, None, None, None
            user = user.split('//')[1]
            password = second_split[2]
        except:
            return False, None, None, None, None, None, None

        try:
            host_port_name_db = second_part_first_split.split('/')
            host_port = host_port_name_db[0]
            if host_port == "":
                return False, None, None, None, None, None, None
            host_ports = host_port.split(':')
            host = host_ports[0]
            if host == "":
                return False, None, None, None, None, None, None
            port = host_ports[1]
            if port == "":
                return False, None, None, None, None, None, None
            name_db = host_port_name_db[1]
            if name_db == "":
                return False, None, None, None, None, None, None
        except:
            return False, None, None, None, None, None, None

        return True, engine, user, password, host, port, name_db

    def check_host_port_receptor(self, string):
        try:
            host_port = string.split(':')
            host = host_port[0]
            if host == "":
                return False, None, None
            port = host_port[1]
            if port == "":
                return False, None, None
        except:
            return False, None, None

        return True, host, port


def change_lat_lng(number):
    """
    Function returns the number in string format and if its negative or
     positive

    :param number: Positive or negative number

    :return: Tuple with number and P or N
    """

    return (
        str(number) if number > 0 else str(number * -1),
        'P' if number > 0 else 'N'
    )


def dictfetchall(result):
    """
    Return all rows from a cursor as a dict"
    """

    columns = [
        'row_id', 'idd', 'timestamp', 'lat', 'lng',
        'speed', 'course', 'altitude', 'satellites'
    ]

    bufferr = [
        dict(zip(columns, row))
        for row in result
    ]

    answer = list()
    for b in bufferr:
        track = {
            'row_id': b['row_id'],
            'idd': b['idd'],
            'date': b['timestamp'].strftime("%d%m%y"),
            'time': b['timestamp'].strftime("%H%M%S"),
            'lat1': change_lat_lng(b['lat'])[0],
            'lat2': change_lat_lng(b['lat'])[1],
            'lng1': change_lat_lng(b['lng'])[0],
            'lng2': change_lat_lng(b['lng'])[1],
            "speed": b['speed'],
            "course": b['course'],
            "altitude": b['altitude'],
            "satellites": b['satellites']
        }
        answer.append(track)

    return answer


def to_utc_str(datetime_with_tz, format='%Y-%m-%d %H:%M:%S'):
    """
    change time zoned date time into utc time.

    :param datetime_with_tz: datetime with time zone
    :param format: string with desired date format

    :return: datime in string format
    """
    datetime_in_utc = datetime_with_tz.astimezone(pytz.utc)

    return datetime_in_utc.strftime(format)


def set_config_db(**data):

    return {
        'db_engine': data['engine'],
        'db_name': data['name'],
        'db_host': data['host'],
        'db_user': data['user'],
        'db_pass': data['pass'],
        'db_port': data['port'],
    }


def get_config_values(config_file):
    """
    Get config values

    :param config_file: config.ini file path

    :return: parsed values found in the config ini file
    """
    database_engine = ConfigSectionMap(config_file, "database")['engine']
    database_name = ConfigSectionMap(config_file, "database")['name']
    database_host = ConfigSectionMap(config_file, "database")['host']
    database_user = ConfigSectionMap(config_file, "database")['user']
    database_pass = ConfigSectionMap(config_file, "database")['pass']
    database_port = ConfigSectionMap(config_file, "database")['port']

    return {
        'db_engine': database_engine,
        'db_name': database_name,
        'db_host': database_host,
        'db_user': database_user,
        'db_pass': database_pass,
        'db_port': database_port,
    }


def get_engine(config_values):
    """
    Takes config values and connects to db engine.

    :param config_values: dictionary containing db connection params and query
    :return: engine, sesssion and WiseTrack object
    """
    Base = automap_base()
    if config_values['db_engine'] == 'debug':
        engine = create_engine('sqlite:///:memory:', echo=True)
    else:
        engine = create_engine('%s://%s:%s@%s:%s/%s' % (
            config_values['db_engine'],
            config_values['db_user'],
            config_values['db_pass'],
            config_values['db_host'],
            config_values['db_port'],
            config_values['db_name'],
        ))

    Base.prepare(engine, reflect=True)
    session = Session(engine)

    return (engine, session)


def required_fields(query):
    forbidden_words = [
        'as date', 'as time',
        'as lat', 'as lng',
        'speed', 'course',
        'altitude', 'satellites'
    ]
    query_words = "".join(
        (char if char.isalpha() else " ") for char in query).split()
    for word in query_words:
        if word.lower() in forbidden_words:
            return word

    return False


def simple_sql_injection_check(query):
    """
    check if query has some kind of sql injection

    :param query: string containg and sql query
    :return: False if is a safe sql select query otherwise invalid word found
    """
    forbidden_words = ['drop', 'insert', 'alter', 'grant', 'truncate', 'alter']
    query_words = "".join(
        (char if char.isalpha() else " ") for char in query).split()
    for word in query_words:
        if word.lower() in forbidden_words:
            return word

    return False


def perform_query(query, engine):
    """
    Perform query in db engine

    :param query: string sql query
    :param engine: sqlanchemy engine object

    :result: list of objects from query execution result
    """

    sql_inject = simple_sql_injection_check(query)
    if sql_inject:
        raise Exception(
            'Forbidden word found in query: {word}'.format(word=sql_inject))
    text_query = text(query)
    conn = engine.connect()
    try:
        result = conn.execute(text_query).fetchall()
    except Exception as e:
        message = 'Error executing query: {error}'.format(error=e)
        logging.error(message)
        print(message)

        return []

    return result


def elements_to_ids(elems):
    vec = [str(e['row_id']) for e in elems]

    return ','.join(vec)


def update_query(table, id_name, state, elements, engine, widget=None):

    sql = "update %s set %s = 1 where %s in (%s)" % (
        table, state, id_name, elements_to_ids(elements))
    print_info("sql update: %s" % sql, widget)
    if elements_to_ids(elements) != "":
        text_query = text(sql)
        conn = engine.connect()
        try:
            result = conn.execute(text_query)
        except Exception as e:
            message = 'Error executing query: {error}'.format(error=e)
            logging.error(message)

            return []

        return result



# -*- coding: utf-8 -*-
import time

from twisted.internet import reactor
from twisted.internet.protocol import ClientFactory

from .echelon_encoder import EchelonEncoder
from utils.db import (
    perform_query,
    dictfetchall,
    update_query,
)

from utils.utils import print_info


class EchelonClientFactory(ClientFactory):

    num_package = 0

    def set_data(
            self, sql_lector_field, engine, table, id_name, field, widget=None):
        self.widget = widget
        self.sql_lector_field = sql_lector_field
        self.engine = engine
        self.table = table
        self.id_name = id_name
        self.field = field
        result = perform_query(self.sql_lector_field, self.engine)
        self.data = dictfetchall(result)
        update_query(table, id_name, field, self.data, self.engine, self.widget)
        self.num_package = self.num_package + 1

    def set_lapse_time(self, lapse_time):
        self.lapse_time = int(lapse_time)

    def clientConnectionFailed(self, connector, reason):
        print_info("Connection Failed", self.widget)
        reactor.stop()

    def clientConnectionLost(self, connector, reason):
        result = perform_query(self.sql_lector_field, self.engine)
        self.data = dictfetchall(result)
        update_query(
            self.table, self.id_name, self.field, self.data, self.engine,
            self.widget)
        print_info(
            "Paquete %s Enviado ------------------------------\n\n" % str(
                self.num_package), self.widget)
        self.num_package = self.num_package + 1
        connector.connect()
        time.sleep(self.lapse_time)
        # if result:
        #     update_query(
        #         self.table, self.id_name, self.field, self.data, self.engine,
        #         self.widget)
        #     print_info(
        #         "Paquete %s Enviado ------------------------------\n\n" % str(
        #             self.num_package), self.widget)
        #     self.num_package = self.num_package + 1
        #     connector.connect()
        #     time.sleep(self.lapse_time)
        # else:
        #     print_info("No hay mas datos para enviar", self.widget)
        #     reactor.stop()

    def buildProtocol(self, addr):
        return EchelonEncoder(self.data, self.widget)



# -*- coding: utf-8 -*-
import configparser


def ConfigSectionMap(config_file, section):
    """
    Handle config section file

    :param config_file: string with file path of config file
    :param section: string, name of the secion inside config file

    :return: dictionary containging params found in config file
    """
    dict1 = {}
    Config = configparser.ConfigParser()
    Config.read(config_file)
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                print(("skip: %s" % option))
        except:
            print(("exception on %s!" % option))
            dict1[option] = None

    return dict1



# -*- coding: utf-8 -*-
STATUS_OK = '1'
STATUS_CONVERTION_ERROR = '2'
STATUS_WRONG_TOKEN = '3'
STATUS_DUPLICATED = '4'
STATUS_INTERNAL_ERROR = '5'

IDENTIFIER_INDEX = 0
TIMESTAMP_INDEX = 1
LATITUDE_INDEX = 2
LONGITUDE_INDEX = 3
COURSE_INDEX = 4
SPEED_INDEX = 5
TRACK_ID = 6
TITLE = "Agente de Comunicación"
LAPSE_TIME_SECONDS = 3
CONEXION_LABEL_TEXT = "Cadena de conexión (motor://usuario:contraseña@host:puerto/nombre_bd)"
SERVER_LABEL_TEXT = "Servidor destino (host:puerto)"
QUERY_LABEL = "Ingresa datos correctos para conexion y consulta"
QUERY_LABEL_FORM = "Ingresa las credenciales para la base de datos"
SQL_LECTOR_TEXT = "Lector SQL"
SQL_UPDATER_TEXT = "Actualizador SQL"
SEND_BUTTON = "Enviar"
UPDATE_BUTTON = "Actualizar"
SAVE_BUTTON = "Guardar"
WARNING_TEXT_SQL = "Debes insertar una sentecia SQL"
WARNING_TITLE = "Atención"
WARNING_CONFIG = "Todos los campos son requeridos"
WARNING_CONFIG_UPDATE = "Todos los campos son requeridos"
WARNING_CONEX = "Cadena de conexión no cumple el formato"
WARNING_HOST_PORT_RECEPTOR = "No cumple el formato host:puerto destino"
ENGINE = "Motor de Base de Datos"
HOST = "Host"
PORT = "Puerto"
NAME = "Nombre de Base de Datos"
USER = "Usuario"
PASSWORD = "Contraseña"
TAB_1 = "Emisor"
TAB_2 = "Configuración Base de Datos"

FAKE_HEADER = {
    "gpsec": "GPSEC",
    "type_package": "DP"
}

FAKE_DATA = [
    {
        "idd": "V9B-939",
        "date": "030518",
        "time": "1000",
        "lat1": "13.424",
        "lat2": "P",
        "lng1": "15.2313",
        "lng2": "N",
        "speed": "40",
        "course": "287",
        "altitude": "3000",
        "satellites": "6"
    },
    {
        "idd": "V9B-939",
        "date": "030518",
        "time": "1000",
        "lat1": "13.424",
        "lat2": "P",
        "lng1": "15.2313",
        "lng2": "N",
        "speed": "40",
        "course": "287",
        "altitude": "3001",
        "satellites": "4"
    },
    {
        "idd": "V9B-939",
        "date": "030518",
        "time": "1001",
        "lat1": "13.424",
        "lat2": "P",
        "lng1": "15.2313",
        "lng2": "N",
        "speed": "40",
        "course": "287",
        "altitude": "3201",
        "satellites": "4"
    }
]

STRING_TITLE = "########### AGENTE DE COMUNICACIONES #############\n\n"
STRING_CONEXION_DB = 'CADENA DE CONEXIÓN (MOTOR://USUARIO:CONTRASEÑA@HOST:PUERTO/NOMBRE_BD): '
STRING_HOST_PORT = 'SERVIDOR DE DESTINO (HOST:PUERTO): '
STRING_LAPSE_TIME = 'LAPSO DE TIEMPO DE ENVIO (SEGUNDOS): '
STRING_CONFIGURATION = '####### Configuración #######'
STRING_CONFIGURATION_SQL = '####### Configuración SQL #######'
STRING_SENT = '####### ENVIO #######'
STRING_WRONG_OPTION = 'Opción incorrecta'
STRING_LESS_DATA = 'Ingrese todos los datos'
STRING_MENU_CONF = '(e)Editar - (g)Guardar - (s)Salir: '
STRING_SQL_SENTENCE = 'SENTENCIA SQL: '
STRING_TABLE_UPDATE = 'TABLA DONDE SE ACTUALIZARÁ EL ESTADO: '
STRING_ID_NAME_UPDATE = 'NOMBRE DEL CAMPO IDENTIFICADOR UNICO EN LA TABLA: '
STRING_FIELD_UPDATE = 'NOMBRE DEL CAMPO DE ESTADO EN LA TABLA: '



# -*- coding: utf-8 -*-
import datetime
import tkinter as tk

def strdate_to_show(strdate):

    return datetime.datetime.strptime(strdate, '%d%m%y').strftime('%d-%m-%Y')


def strtime_to_show(strtime):

    return datetime.datetime.strptime(strtime, '%H%M%S').strftime('%H:%M:%S')


def print_info(msg, widget):
    if widget:
        widget.insert(tk.INSERT, msg+'\n')
    else:
        print(msg)



# -*- coding: utf-8 -*-
import uuid

from twisted.internet.protocol import Protocol
from utils.constants import FAKE_HEADER
from utils.utils import strdate_to_show, strtime_to_show, print_info


class EchelonEncoder(Protocol):

    gpsec = None
    idd = None
    type_package = None
    message = ''
    date = ''
    time = ''
    time = ''
    lat1 = ''
    lat2 = ''
    lng1 = ''
    lng2 = ''
    speed = ''
    course = ''
    altitude = ''
    satellites = ''
    data = dict()

    def __init__(self, data, widget=None):
        self.data = data
        self.widget = widget

    def set_header(self, **data):
        self.gpsec = data['gpsec']
        self.type_package = data['type_package']

    def set_message(self, **data):
        self.idd = data['idd']
        self.date = data['date']
        self.time = data['time']
        self.lat1 = data['lat1']
        self.lng1 = data['lng1']
        self.lat2 = data['lat2']
        self.lng2 = data['lng2']
        self.speed = int(data['speed'])
        self.course = int(data['course'])
        self.altitude = int(data['altitude'])
        self.satellites = int(data['satellites'])

    def create_uuid(self):
        self.uuid = uuid.uuid4()

    def create_message(self):
        self.create_uuid()
        self.message = '{0};{1};{2};{3};{4};{5};{6};{7};{8};{9}'.format(
            self.date,
            self.time,
            self.lat1,
            self.lat2,
            self.lng1,
            self.lng2,
            self.speed,
            self.course,
            self.altitude,
            self.satellites,
            # self.uuid,
        )

    def create(self):
        self.create_message()

        return '{0}#{1}#{2}#{3}\r\n'.format(
            self.gpsec,
            self.idd,
            self.type_package,
            self.message
        )

    def print_header_fields(self):
        return '{0:10}{1:15}{2:15}{3:15}{4:15}{5:5}\n{6:-^80s}'.format(
            'Placa',
            'Fecha',
            'Hora',
            'Latitud',
            'Longitud',
            'Velocidad',
            '-',
        )

    def show_message(self):

        return '{0:10}{1:15}{2:15}{3:15}{4:15}{5:5} km'.format(
            self.idd,
            strdate_to_show(self.date),
            strtime_to_show(self.time),
            self.lat1,
            self.lng1,
            self.speed,
        )

    def send_message(self):
        header = {
            "gpsec": FAKE_HEADER["gpsec"] or self.gpsec,
            "type_package": FAKE_HEADER["type_package"] or self.type_package
        }

        messages = list()
        for element in self.data:
            self.set_header(**header)
            self.set_message(**element)
            messages.append((self.show_message(), self.create().encode()))

        return messages

    def connectionMade(self):
        print_info(self.print_header_fields(), self.widget)
        for msg, enconded_msg in self.send_message():
            print("msg: ", msg)
            print_info(msg, self.widget)
            self.transport.write(enconded_msg)
        self.transport.loseConnection()

    def connectionLost(self, reason):
        print_info("Enviando ...", self.widget)



# -*- coding: utf-8 -*-
import json
import tkinter as tk
import tkinter.scrolledtext as tkst
from tkinter import messagebox
from twisted.internet import reactor

from classes.echelon_agent import EchelonAgent

from classes.echelon_client_factory import EchelonClientFactory
from utils.db import (
    get_engine,
    perform_query,
    DbCredentials,
    set_config_db,
    dictfetchall
)
from utils.constants import *


class Application(tk.Frame):
    conexion_label = None

    def __init__(self, master=None):
        self.db_cred = DbCredentials()

        data_db = self.db_cred.get()
        self.conexion_db = data_db['str_conexion']
        self.host_port = data_db['server_receptor']
        self.lapse_time = data_db['lapse_seconds']
        self.sentence_sql = data_db['sql_lector']
        self.table_update = data_db['table_update']
        self.id_name_update = data_db['id_name_update']
        self.field_update = data_db['field_update']

        self.master = master
        super().__init__(master)

        self.create_widgets()
        self.fill_fields()
        self.factory = EchelonClientFactory()

    def save_total_conf_db(self):
        data_save = {
            'str_conexion': self.conexion_field.get('1.0', 'end').strip(),
            'server_receptor': self.server_field.get('1.0', 'end').strip(),
            'lapse_seconds': self.lapse_time_field.get('1.0', 'end').strip(),
            'sql_lector': self.sql_lector_field.get('1.0', 'end').strip(),
            'table_update': self.table_update_field.get('1.0', 'end').strip(),
            'id_name_update': self.id_update_field.get('1.0', 'end').strip(),
            'field_update': self.state_update_field.get('1.0', 'end').strip(),
        }
        self.db_cred.insert(**data_save)

    def create_widgets(self):
        self.master.title(TITLE)
        self.master.geometry("1000x860")

        self.query_label = tk.Label(
            self, text=QUERY_LABEL, anchor=tk.W,
            font='Helvetica 18 bold').grid(row=1, column=0, columnspan=2)

        self.conexion_label = tk.Label(
            self, text=STRING_CONEXION_DB, height=3).grid(row=2, column=0)

        self.server_label = tk.Label(
            self, text=STRING_HOST_PORT, height=3).grid(row=2, column=1)

        self.conexion_field = tk.Text(self, width=50, height=1)
        self.conexion_field.grid(row=3, column=0)

        self.server_field = tk.Text(self, width=50, height=1)
        self.server_field.grid(row=3, column=1)

        self.sql_lector_label = tk.Label(
            self, text=STRING_SQL_SENTENCE, height=3).grid(
            row=4, column=0, columnspan=2)
        self.sql_lector_field = tk.Text(self, width=120, height=10)
        self.sql_lector_field.grid(row=5, column=0, columnspan=2)

        self.label_lapse_time = tk.Label(
            self, text=STRING_LAPSE_TIME, height=3).grid(row=6, column=0)
        self.label_table_update = tk.Label(
            self, text=STRING_TABLE_UPDATE, height=3).grid(row=6, column=1)

        self.lapse_time_field = tk.Text(self, width=60, height=1)
        self.lapse_time_field.grid(row=7, column=0)
        self.table_update_field = tk.Text(self, width=60, height=1)
        self.table_update_field.grid(row=7, column=1)

        self.label_id_update = tk.Label(
            self, text=STRING_ID_NAME_UPDATE, height=3).grid(row=8, column=0)
        self.id_update_field = tk.Text(self, width=60, height=1)
        self.id_update_field.grid(row=9, column=0)

        self.label_state_update = tk.Label(
            self, text=STRING_FIELD_UPDATE, height=3).grid(row=8, column=1)
        self.state_update_field = tk.Text(self, width=60, height=1)
        self.state_update_field.grid(row=9, column=1)

        self.send_button = tk.Button(
            self, text=SEND_BUTTON, fg="blue",
            command=self.send_query)
        self.send_button.grid(
            row=11, column=0, columnspan=2, pady=20, ipadx=40, ipady=10)

        self.logger_field = tkst.ScrolledText(
            master=self,
            wrap=tk.WORD,
            width=120,
            height=20,
        )
        self.logger_field.grid(
            row=12, column=0, columnspan=2, pady=20)

        self.pack()

    def fill_fields(self):
        self.conexion_field.insert('1.0', self.conexion_db)
        self.server_field.insert('1.0', self.host_port)
        self.sql_lector_field.insert('1.0', self.sentence_sql)
        self.lapse_time_field.insert('1.0', self.lapse_time)

        self.table_update_field.insert('1.0', self.table_update)
        self.id_update_field.insert('1.0', self.id_name_update)
        self.state_update_field.insert('1.0', self.field_update)

    def check_require_fields(self):
        fields = list()
        fields.append(self.conexion_field.get('1.0', 'end').strip())
        fields.append(self.server_field.get('1.0', 'end').strip())
        fields.append(self.sql_lector_field.get('1.0', 'end').strip())
        fields.append(self.lapse_time_field.get('1.0', 'end').strip())
        fields.append(self.table_update_field.get('1.0', 'end').strip())
        fields.append(self.id_update_field.get('1.0', 'end').strip())
        fields.append(self.state_update_field.get('1.0', 'end').strip())

        for field in fields:
            if field == "":
                return False
        return True

    def check_require_fields_update(self):
        conexion_field = self.conexion_field.get('1.0', 'end').strip()
        server_field = self.server_field.get('1.0', 'end').strip()
        sql_updater_field = self.sql_updater_field.get('1.0', 'end').strip()
        if(conexion_field == "" or server_field == "" or
                sql_updater_field == ""):
            return False

        return True

    def update_query(self):
        if self.check_require_fields_update():
            pass
        else:
            messagebox.showwarning(WARNING_TITLE, WARNING_CONFIG_UPDATE)

    def send_query(self):
        self.save_total_conf_db()
        self.logger_field.insert('1.0', "Enviando...\n")

        def send():
            conexion_field = self.conexion_field.get('1.0', 'end').strip()
            server_field = self.server_field.get('1.0', 'end').strip()
            sql_lector_field = self.sql_lector_field.get('1.0', 'end').strip()
            lapse_time_field = self.lapse_time_field.get('1.0', 'end').strip()

            table = self.table_update_field.get('1.0', 'end').strip()
            id_name = self.id_update_field.get('1.0', 'end').strip()
            field = self.state_update_field.get('1.0', 'end').strip()

            is_correct, engine, user, password, host, port, name_db = (
                self.db_cred.check_string_con(conexion_field)
            )

            if is_correct:
                config_values = set_config_db(**{
                    'engine': engine,
                    'name': name_db,
                    'host': host,
                    'user': user,
                    'pass': password,
                    'port': port,
                })
                engine, session = get_engine(config_values)
                self.logger_field.insert(
                    tk.INSERT, "Conectado a Base de Datos local\n")
            else:
                messagebox.showwarning(WARNING_TITLE, WARNING_CONEX)

            is_correct, host, port = self.db_cred.check_host_port_receptor(
                server_field)

            if is_correct:
                self.factory.set_data(
                    sql_lector_field, engine, table, id_name, field,
                    self.logger_field)
                self.factory.set_lapse_time(lapse_time_field)

                reactor.connectTCP(host, int(port), self.factory)
                reactor.run()
            else:
                messagebox.showwarning(
                    WARNING_TITLE, WARNING_HOST_PORT_RECEPTOR)

        if self.check_require_fields():
            self.send_button.config(state="disabled")
            root.after(1000, send)

        else:
            messagebox.showwarning(WARNING_TITLE, WARNING_CONFIG)


if __name__ == '__main__':

    root = tk.Tk()
    app = Application(master=root)
    app.mainloop()



# -*- coding: utf-8 -*-
import os
import logging
import argparse
import sys
from utils.config_loader import ConfigSectionMap
from classes.echelon_agent import EchelonAgent


def get_config_values(config_file):
    echelon_db = ConfigSectionMap(config_file, "echelon")['db']
    echelon_server_receptor = ConfigSectionMap(
        config_file, "echelon")['server_receptor']
    echelon_sql_lector = ConfigSectionMap(config_file, "echelon")['sql_lector']
    echelon_lapse_seconds = ConfigSectionMap(
        config_file, "echelon")['lapse_seconds']
    echelon_table_update = ConfigSectionMap(
        config_file, "echelon")['table_update']
    echelon_id_field_update = ConfigSectionMap(
        config_file, "echelon")['id_field_update']
    echelon_state_field_update = ConfigSectionMap(
        config_file, "echelon")['state_field_update']

    return {
        'echelon_db': echelon_db,
        'echelon_server_receptor': echelon_server_receptor,
        'echelon_sql_lector': echelon_sql_lector,
        'echelon_lapse_seconds': echelon_lapse_seconds,
        'echelon_table_update': echelon_table_update,
        'echelon_id_field_update': echelon_id_field_update,
        'echelon_state_field_update': echelon_state_field_update,
    }


def echelon_client(script_path, config_file):
    config_values = get_config_values(config_file)
    echelon_agent = EchelonAgent()
    echelon_agent.set_values(**config_values)
    echelon_agent.send_query(
        config_values['echelon_table_update'],
        config_values['echelon_id_field_update'],
        config_values['echelon_state_field_update']
    )


if __name__ == '__main__':

    script_path = os.path.abspath(os.path.dirname(sys.argv[0])) + '/'

    if not os.path.exists(script_path + 'logs'):
        try:
            os.makedirs(script_path + 'logs')
        except OSError as exc:
            pass

    if not os.path.exists(script_path + 'conf'):
        try:
            os.makedirs(script_path + 'conf')
        except OSError as exc:
            pass

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-a',
        '--author',
        action='version',
        help="show author's info and exit",
        version='Eduardo Chauca Gallegos <eduardo@inka-labs.com>')

    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s 0.1.0')

    parser.add_argument(
        '-c',
        '--config_file',
        action="store",
        dest="config_file",
        help="configuration file path",
        default=script_path + 'conf/config.ini')

    parser.add_argument(
        '-l',
        '--log_file',
        action="store",
        dest="log_file",
        help="logging file path",
        default=script_path + "logs/echelon_command.log")

    args = parser.parse_args()
    logging.basicConfig(
        filename=args.log_file,
        format='%(levelname)s : %(asctime)s :: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        level=logging.DEBUG
    )

    echelon_client(script_path, args.config_file)



# -*- coding: utf-8 -*-

from utils.db import (
    get_engine,
    DbCredentials,
    set_config_db,
)
from classes.echelon_client_factory import EchelonClientFactory
from twisted.internet import reactor


class EchelonAgent:
    connection_field = ''
    server_field = ''
    sql_lector_field = ''

    def __init__(self):
        self.db_cred = DbCredentials()
        self.factory = EchelonClientFactory()

    def set_values(self, **data):
        self.conexion_field = data['echelon_db']
        self.server_field = data['echelon_server_receptor']
        self.sql_lector_field = data['echelon_sql_lector']
        self.echelon_lapse_seconds = data['echelon_lapse_seconds']

    def check_require_fields(self):
        if(self.conexion_field == "" or self.server_field == "" or
                self.sql_lector_field == ""):
            return False

        return True

    def send_query(self, table='', id_name='', field=''):
        if self.check_require_fields():
            is_correct, engine, user, password, host, port, name_db = (
                self.db_cred.check_string_con(self.conexion_field)
            )

            print(
                "engine, user, password, host, port, name_db: ",
                engine, user, password, host, port, name_db
            )

            if is_correct:
                self.db_values = set_config_db(**{
                    'engine': engine,
                    'name': name_db,
                    'host': host,
                    'user': user,
                    'pass': password,
                    'port': port,
                })
                self.engine, self.session = get_engine(self.db_values)
                print("... Conectado a Base de Datos local\n")
            else:
                print("Error")

            is_correct, host, port = self.db_cred.check_host_port_receptor(
                self.server_field)
            if is_correct:
                self.factory.set_data(
                    self.sql_lector_field, self.engine, table, id_name, field)
                self.factory.set_lapse_time(self.echelon_lapse_seconds)
                reactor.connectTCP(host, int(port), self.factory)
                reactor.run()
            else:
                print("Error")

        else:
            print("Error")



# -*- coding: utf-8 -*-
from classes.echelon_agent import EchelonAgent
from classes.echelon_console import EchelonConsole


def two_steps(wizard, option):

    while option != '3':
        if option == "1":
            wizard.show_conf_step()
            option_conf = wizard.show_sub_menu()
            option_conf = wizard.wait_editable_option(option_conf)

            while option_conf == 'e':
                option_conf = wizard.inside_conf()

            if option_conf == 'g':
                # wizard.save_conf()
                option = wizard.show_main_menu()
            if option_conf == 's':
                option = wizard.show_main_menu()

        if option == "2":
            wizard.show_sql_step()
            option_sql = wizard.show_sub_menu()
            option_sql = wizard.wait_editable_option(option_sql)

            while option_sql == 'e':
                option_sql = wizard.inside_sql()

            if option_sql == 'g':
                # wizard.save_sql()
                option = wizard.show_main_menu()
            if option_sql == 's':
                option = wizard.show_main_menu()

    return option


if __name__ == '__main__':

    echelon = EchelonAgent()
    wizard = EchelonConsole(echelon.db_cred)
    option = wizard.show_main_menu()
    option = two_steps(wizard, option)

    if option == '3':
        print(wizard.check_confs_sqls_data())
        while wizard.check_confs_sqls_data() is False:
            option = wizard.show_main_menu(wizard.string_less_data)
            option = two_steps(wizard, option)

        wizard.inside_send()
        data = {
            'echelon_db': wizard.conexion_db,
            'echelon_server_receptor': wizard.host_port,
            'echelon_sql_lector': wizard.sentence_sql,
            'echelon_lapse_seconds': wizard.lapse_time,
        }
        echelon.set_values(**data)
        wizard.save_total_conf_db()
        echelon.send_query(
            wizard.table_update,
            wizard.id_name_update,
            wizard.field_update
        )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="echelon_client_agent",
    version="0.0.3",
    author="Inka-labs",
    author_email="eduardo@inka-labs.com",
    description="Communication Agent",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/connecttix/echelon-client",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        "attrs==17.4.0",
        "Automat==0.6.0",
        "certifi==2018.4.16",
        "chardet==3.0.4",
        "constantly==15.1.0",
        "dicttoxml==1.7.4",
        "hyperlink==18.0.0",
        "idna==2.6",
        "incremental==17.5.0",
        "pytz==2017.3",
        "requests==2.18.4",
        "six==1.11.0",
        "SQLAlchemy==1.2.1",
        "Twisted==18.4.0",
        "urllib3==1.22",
        "zope.interface==4.5.0",
    ],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



# -*- coding: utf-8 -*-
import os

from utils.constants import (
    STRING_TITLE,
    STRING_CONEXION_DB,
    STRING_HOST_PORT,
    STRING_LAPSE_TIME,
    STRING_CONFIGURATION,
    STRING_CONFIGURATION_SQL,
    STRING_SENT,
    STRING_WRONG_OPTION,
    STRING_MENU_CONF,
    STRING_SQL_SENTENCE,
    STRING_TABLE_UPDATE,
    STRING_ID_NAME_UPDATE,
    STRING_FIELD_UPDATE,
    STRING_LESS_DATA,
)


class EchelonConsole:

    string_title = STRING_TITLE
    string_conexion_db = STRING_CONEXION_DB
    string_host_port = STRING_HOST_PORT
    string_lapse_time = STRING_LAPSE_TIME
    string_configuration = STRING_CONFIGURATION
    string_configuration_sql = STRING_CONFIGURATION_SQL
    string_sent = STRING_SENT
    string_wrong_option = STRING_WRONG_OPTION
    string_menu_conf = STRING_MENU_CONF
    string_sql_sentence = STRING_SQL_SENTENCE
    string_table_update = STRING_TABLE_UPDATE
    string_id_name_update = STRING_ID_NAME_UPDATE
    string_field_update = STRING_FIELD_UPDATE
    string_less_data = STRING_LESS_DATA
    conexion_db = ''
    host_port = ''
    lapse_time = ''
    sentence_sql = ''
    table_update = ''
    id_name_update = ''
    field_update = ''

    def __init__(self, db_object):
        self.db_object = db_object
        data_db = self.db_object.get()
        self.conexion_db = data_db['str_conexion']
        self.host_port = data_db['server_receptor']
        self.lapse_time = data_db['lapse_seconds']
        self.sentence_sql = data_db['sql_lector']
        self.table_update = data_db['table_update']
        self.id_name_update = data_db['id_name_update']
        self.field_update = data_db['field_update']

    def save_total_conf_db(self):
        data_save = {
            'str_conexion': self.conexion_db,
            'server_receptor': self.host_port,
            'lapse_seconds': self.lapse_time,
            'sql_lector': self.sentence_sql,
            'table_update': self.table_update,
            'id_name_update': self.id_name_update,
            'field_update': self.field_update,
        }
        self.db_object.insert(**data_save)

    def save_sql(self):
        pass

    def print_main_menu(self):
        conf_conf = "(1) Configuración" if (
            self.conexion_db == "" or self.host_port == "" or
            self.lapse_time == "") else "(1) Configuración (LISTO)"

        conf_sql = "(2) Ingresar SQL" if (
            self.sentence_sql == "" or self.table_update == "" or
            self.id_name_update == "" or
            self.field_update == "") else "(2) Ingresar SQL (LISTO)"

        print(conf_conf)
        print(conf_sql)
        print("(3) Enviar\n")

    def main_options(self, option):
        while option != "1" and option != "2" and option != "3":
            os.system('clear')
            self.print_main_title()
            self.print_main_menu()
            print("Opción incorrecta")
            option = input("Opción: ")

        return option

    def print_main_title(self):
        print(self.string_title)

    def show_main_menu(self, error=''):
        os.system('clear')
        self.print_main_title()
        self.print_main_menu()
        print(error)
        option = input("Opción: ")
        option = str(option).strip()

        return self.main_options(option)

    def show_sub_menu(self):

        return input(self.string_menu_conf)

    def edit_conf(self):
        os.system('clear')
        self.print_main_title()
        print(self.string_configuration)
        print("\n")
        self.conexion_db = input(self.string_conexion_db)
        self.host_port = input(self.string_host_port)
        self.lapse_time = input(self.string_lapse_time)

    def edit_sql(self):
        os.system('clear')
        self.print_main_title()
        print(self.string_configuration_sql)
        print("\n")
        self.sentence_sql = input(self.string_sql_sentence)
        print("\n ---- Datos para la actualización ---- \n")
        self.table_update = input(self.string_table_update)
        self.id_name_update = input(self.string_id_name_update)
        self.field_update = input(self.string_field_update)

    def show_conf_step(self):
        os.system('clear')
        self.print_main_title()
        print(self.string_configuration)
        print('\n{0}{1}'.format(self.string_conexion_db, self.conexion_db))
        print('{0}{1}'.format(self.string_host_port, self.host_port))
        print('{0}{1}\n'.format(self.string_lapse_time, self.lapse_time))

    def show_sql_step(self):
        os.system('clear')
        self.print_main_title()
        print(self.string_configuration_sql)
        print('\n{0}{1}'.format(self.string_sql_sentence, self.sentence_sql))
        print("\n ---- Datos para la actualización ---- \n")
        print('{0}{1}'.format(self.string_table_update, self.table_update))
        print('{0}{1}'.format(self.string_id_name_update, self.id_name_update))
        print('{0}{1}\n'.format(self.string_field_update, self.field_update))

    def wait_editable_option(self, option):
        while option != "e" and option != "g" and option != "s":
            os.system('clear')
            self.print_main_title()
            print(self.string_configuration)
            self.show_conf_step()
            print(self.string_wrong_option)
            option = input(self.string_menu_conf)

        return option

    def inside_conf(self):
        self.edit_conf()
        self.show_conf_step()

        return self.show_sub_menu()

    def inside_sql(self):
        self.edit_sql()
        self.show_sql_step()

        return self.show_sub_menu()

    def inside_send(self):
        os.system("clear")
        self.print_main_title()
        print(self.string_sent)
        print("Conectando a Base de datos local ... ")

    def check_confs_sqls_data(self):
        if (
            self.conexion_db == "" or self.host_port == "" or
            self.lapse_time == "" or self.sentence_sql == "" or
            self.table_update == "" or self.id_name_update == "" or
                self.field_update == ""):
            return False

        return True
